/***************************************************************************
                          animatedobjects.cpp  -  description
                             -------------------
    begin                : Tue Jul 30 2002
    copyright            : (C) 2002 by Eugen C.
    email                : eug@thekompany.com
 ***************************************************************************/
 
#include "animatedobjects.h"
#include "decorobjects.h"
#include "qiconloader.h"

#include <qimage.h>
#include <qbitmap.h>

#include <stdlib.h>

/*******************************************************  
                          _____ _                      
                         |  ___(_) __ _ _   _ _ __ ___ 
                         | |_  | |/ _` | | | | '__/ _ \
                         |  _| | | (_| | |_| | | |  __/
                         |_|   |_|\__, |\__,_|_|  \___|
                                  |___/                

********************************************************/
KGrFigure :: KGrFigure(int x, int y)
  : KGrObj(),
    map(0L)
{
  this->xpos = mem_x = x;
  this->ypos = mem_y = y;
  
  relx = mem_relx = 0;
  rely = mem_rely = 0;
  
  absx = x * objectSize;
  absy = y * objectSize;

  nuggets = 0;
  status_ = KGrObj::STANDING;
  
  walkTimer = new QTimer (this);
  fallTimer = new QTimer (this);
}

KGrFigure :: ~KGrFigure ()
{
}

int KGrFigure::herox =0;
int KGrFigure::heroy =0;

int KGrFigure::getx()
{
  return absx;
}

int KGrFigure::gety()
{
  return absy;
}

void KGrFigure::init(int a,int b)
{
  walkTimer->stop();
  fallTimer->stop();
  
  xpos = mem_x = a;
  ypos = mem_y = b;
  
  relx = mem_relx = 0;
  rely = mem_rely = 0;
  
  nuggets = 0;
  status_ = KGrObj::STANDING;
}

void KGrFigure::setNuggets(int n)
{
  nuggets = n;
}

KGrObj::Status KGrFigure::status()
{
  return status_;
}

KGrObj::Direction KGrFigure::direction()
{
  return direction_;
}

bool KGrFigure::canWalkRight()
{
  bool bRet=true;
  
  switch( map->playField[xpos+1][ypos]->whatIam() )
  {
    case BRICK :
    case BETON :
    case FBRICK :
      bRet = false;
      break;
    default :
      bRet = true;
      break;
  }
  
  return bRet;
}

bool KGrFigure::canWalkLeft()
{
  bool bRet=true;
  
  switch( map->playField[xpos-1][ypos]->whatIam() )
  {
    case BRICK :
    case BETON :
    case FBRICK :
      bRet = false;
      break;
    default :
      bRet = true;
      break;
  }
  
  return bRet;
}

bool KGrFigure::canWalkUp()
{
  bool bRet=true;
  
  if( LADDER == map->playField[xpos][ypos]->whatIam() )
  {
    switch( map->playField[xpos][ypos-1]->whatIam() )
    {
      case BRICK :
      case BETON :
      case FBRICK :
        bRet = false;
        break;
      default :
        bRet = true;
        break;
    }
    
    return bRet;
  }
  
  return false;
}

bool KGrFigure::canWalkDown()
{
  bool bRet=true;
  
  if( LADDER == map->playField[xpos][ypos]->whatIam() )
  {
    switch( map->playField[xpos][ypos+1]->whatIam() )
    {
      case BRICK :
      case BETON :
      case FBRICK :
        bRet = false;
        break;
      default :
        bRet = true;
        break;
    }
    
    return bRet;
  }
  else if( LADDER == map->playField[xpos][ypos+1]->whatIam() )
    return true;
  
  return false;
}

bool KGrFigure::canStand()
{
  bool bRet=true;
  
  switch( map->playField[xpos][ypos+1]->whatIam() )
  {
    case BRICK :
    case BETON :
    case FBRICK :
    case LADDER :
      bRet = true;
      break;
    default :
      if( LADDER == map->playField[xpos][ypos]->whatIam() ||
          standOnEnemy() )
        bRet = true;
      else
        bRet = false;
      break;
  }
  
  return bRet;
}

bool KGrFigure::hangAtPole()
{ 
  return (map->playField[xpos][ypos]->whatIam() == POLE);
}

void KGrFigure::walkUp()
{
  actualPixmap = (actualPixmap == KGrObj::CLIMB1) ? KGrObj::CLIMB2 : KGrObj::CLIMB1;
  if( walkCounter++ < MAX_STEPS )
  { 
    if( canWalkUp() )
    {
      rely -= STEP;
      absy -= STEP;
    }
    
    walkTimer->start(WALKDELAY, TRUE);
  } 
  else
  { 
    if( canWalkUp() )
    {
      ypos--;
      rely = 0;
      absy = objectSize * ypos;
      walkCounter = 0;
    }
    
    status_ = KGrObj::STANDING;
  }
}

void KGrFigure::walkDown()
{
  if( hangAtPole() )
    initFall(19);
  else
  {
    actualPixmap = (actualPixmap == KGrObj::CLIMB1) ? KGrObj::CLIMB2 : KGrObj::CLIMB1;
    if( walkCounter++ < MAX_STEPS )
    { 
      if( canWalkDown() )
      {
        rely += STEP;
        absy += STEP;
      }
      
      walkTimer->start(WALKDELAY, TRUE);
    } 
    else 
    { 
      if( canWalkDown() )
      {
        ypos++;
        rely = 0;
        absy = objectSize*ypos;
        walkCounter=0;
      }
      
      if( !canStand() ) // kein Halt....urgs
        initFall(19);
      else 
        status_ = KGrObj::STANDING;
    }
  }
}

void KGrFigure::walkLeft()
{
  if (walkCounter++)
  {
    if(++actualPixmap % 4)
    {
      if (canWalkLeft())
      {
        relx -= STEP;
        absx -= STEP;
      }
      
      walkTimer->start (WALKDELAY, TRUE);
    }
    else
    {
      actualPixmap -= 4;
      if( canWalkLeft() )
      {
        xpos--;
        relx = 0;
        absx = xpos * objectSize;
      }
      if( !(canStand() || hangAtPole()) ) // falling
        initFall(18);
      else
        status_ = KGrObj::STANDING;
    }
    
    if(relx == -8)
    {
      if( map->playField[xpos-1][ypos]->whatIam() == POLE )
        actualPixmap = 14;
      else
        actualPixmap = 6;
    }
  }
  else
    status_ = KGrObj::STANDING;
}

void KGrFigure::walkRight()
{
  if( walkCounter++ ) // wenn 0, soll sich Figur nur umdrehen
  {
    if(++actualPixmap % 4) // wenn true, dann ist kein vollstndiger Schritt gemacht
    {
      if( canWalkRight() ) 
      {
        relx += STEP;
        absx += STEP; // nur vorwrts gehen, wenn es auch mglich ist 
        if(relx == 8) // Figur ist die Hlfte. Soll sie eine
        {
          if( hangAtPole() )  // Pole anfassen oder loslassen ist jetzt
            actualPixmap = 10; //  der richtige Zeipunkt
          else
            actualPixmap = 2;
        }
      }
      walkTimer->start (WALKDELAY, TRUE); // neues Delay
    } 
    else
    {
      actualPixmap -= 4; // Schritt war vollendet
      
      if( canWalkRight() )
      {
        xpos++;
        relx = 0;
        absx = objectSize*xpos;
      } //gehe entgltig nach rechts
      
      if( !(canStand() || hangAtPole()) ) // kein Halt mehr...arrrgghhh
        initFall (19);
      else 
        status_ = KGrObj::STANDING;
    } // Figur hat Halt  
  }
  else
    status_ = KGrObj::STANDING; // Figur sollte sich nur Umdrehen.
}

void KGrFigure::eraseOldFigure()
{
  map->playField[mem_x][mem_y]->eraseFigure (mem_relx, mem_rely);
  
  if (mem_relx != 0)
    if (mem_relx < 0)
      map->playField[mem_x-1][ypos]->eraseFigure (mem_relx + objectSize, mem_rely);
    else
      map->playField[mem_x+1][ypos]->eraseFigure (mem_relx - objectSize, mem_rely);
  else if (mem_rely != 0)
    if (mem_rely < 0)
      map->playField[xpos][mem_y-1]->eraseFigure (mem_relx, mem_rely + objectSize);
    else
      map->playField[xpos][mem_y+1]->eraseFigure (mem_relx, mem_rely - objectSize);
}

void KGrFigure::initFall(int apm)
{
  status_ = KGrObj::FALLING;
  actualPixmap = apm;
  walkCounter=1;
  walkTimer->stop();
  fallTimer->start(FALLDELAY, TRUE);
}

void KGrFigure::showFigure ()
{
}

void KGrFigure::setPlayfield(FieldMap * fieldMap)
{
  map = fieldMap;
}

/*******************************************************
                             _   _                
                            | | | | ___ _ __ ___  
                            | |_| |/ _ \ '__/ _ \ 
                            |  _  |  __/ | | (_) |
                            |_| |_|\___|_|  \___/ 

********************************************************/                 
KGrHero :: KGrHero (int xpos, int ypos)
  :KGrFigure(xpos, ypos)
{
  QString name;
  
  status_ = KGrObj::STANDING;
  actualPixmap = 18;
  WALKDELAY = 45;
  FALLDELAY = 50;
    
  for (int i = 0; i < 20; i++)
  {
    name = QString("hero_") + QString::number(i);
    heroPm[i] = QIconLoader::instance()->image( name );
    if( heroPm[i].isNull() )
      qDebug("Hmm, hero error!");
  }

  herox=xpos;
  heroy=ypos;
  
  started = false;
  
  connect (walkTimer, SIGNAL (timeout ()), SLOT (walkTimeDone ()));
  connect (fallTimer, SIGNAL (timeout ()), SLOT (fallTimeDone ()));
}

KGrHero::~KGrHero()
{

  delete walkTimer;
  delete fallTimer;
}

/* Es ist Notwendig der eigentlichen Timerfunktion diese
   Startwalk vorzuschalten, um bei einem evtl. notwendigem
   Richtungswechsel der Figur diese Bewegung mit einzufgen */
void KGrHero::startWalk ()
{
  switch (nextDir)
  {
    case KGrObj::UP: 
      if( map->playField[xpos][ypos]->whatIam () == LADDER )
      {
        walkCounter = 1;
        direction_ = KGrObj::UP;
      }
      break;
    case KGrObj::RIGHT:
      if( hangAtPole() )
        actualPixmap = KGrObj::RIGHTCLIMB1;
      else
        actualPixmap = KGrObj::RIGHTWALK1;
        
      if( direction_ != KGrObj::RIGHT )
        walkCounter = 0;
      else
        walkCounter = 1;
      
      direction_ = KGrObj::RIGHT;
      break;
    case KGrObj::DOWN:
      if( (map->playField[xpos][ypos]->whatIam () == LADDER) ||
          (map->playField[xpos][ypos+1]->whatIam () == LADDER) )
      {
        walkCounter = 1;
        direction_ = KGrObj::DOWN;
      }
      // wenn figur an Stange haengt und nichts unter ihr,
      // dann soll sie fallen
      else if( hangAtPole() && (!canStand()) )
      {
        status_ = KGrObj::STANDING;
        actualPixmap = (direction_ == KGrObj::RIGHT) ? 19 : 18;
        walkCounter = 1;
        direction_ = KGrObj::STAND;
        walkTimer->stop();
        // fallTimer->start(FALLDELAY,TRUE);
      }
      break;
    case KGrObj::LEFT:
      if( hangAtPole() )
        actualPixmap = KGrObj::LEFTCLIMB1;
      else
        actualPixmap = KGrObj::LEFTWALK1;
      if( direction_ != KGrObj::LEFT )
        walkCounter = 0;
      else
        walkCounter = 1;
      direction_ = KGrObj::LEFT;
      break;
    default : 
      direction_ = KGrObj::STAND; 
      status_ = KGrObj::FALLING; 
      break;
  }
    
  nextDir = KGrObj::STAND;
  if( status_ != KGrObj::FALLING )//immer ausfhren, ausser beim fallen
  {
    status_ = KGrObj::WALKING; // da sonst der FALLINGstatus wieder
    showFigure ();      // geaendert wird und der falsche Timer 
    // walkTimer->start (WALKDELAY, TRUE);} // anspringt.
  }
}

void KGrHero::setKey(KGrObj::Direction key)
{
  nextDir = key;
}

void KGrHero::init(int a,int b)
{
  walkTimer->stop();
  fallTimer->stop();

  xpos = mem_x = a;
  ypos = mem_y = b;
  relx = mem_relx = 0;
  rely = mem_rely = 0;
 
  absx = objectSize*xpos;
  absy = objectSize*ypos;

  nuggets = 0;
  
  herox=xpos;
  heroy=ypos;

  actualPixmap=19;
}

void KGrHero::start()
{
  if( !(canStand() || hangAtPole()) ) // Held muss wohl fallen...
  {
    status_ = KGrObj::FALLING;
    fallTimeDone();
  }
  else 
  {
    status_ = KGrObj::STANDING;
    walkTimeDone();
  }
}

void KGrHero::walkTimeDone ()
{
  if ((status_ == KGrObj::STANDING) && (nextDir != KGrObj::STAND))
    startWalk();
  if (status_ != KGrObj::STANDING)
  {
    switch (direction_)
    {
      case KGrObj::UP:
        if( (ypos==1) && (nuggets<=0) )
          leaveLevel();
        else 
          walkUp();
        break;
      case KGrObj::DOWN:
        walkDown();
        break;
      case KGrObj::RIGHT:
        walkRight();
        break;
      case KGrObj::LEFT:
        walkLeft();
        break;
      default : 
        if( !canStand() || hangAtPole() ) // falling
          initFall(18);
        else 
          status_ = KGrObj::STANDING;
        break;
      }
      
    herox=xpos;
    heroy=ypos; // Koordinatenvariablen neu 
    // wenn Held genau ein Feld weitergelaufen ist,
    if ((relx==0)&&(rely==0)) // dann setzte statische 
    { 
      collectNugget(); // und nehme evtl. Nugget
    }
    
    showFigure(); 
  }
  
  if( status_ == KGrObj::STANDING )
  {
    if( !canStand() && !hangAtPole() )
      initFall(18);
    else
      walkTimer->start(WALKDELAY, TRUE);
  }
  
  if( isInEnemy() )
    caughtHero();
}

void KGrHero::fallTimeDone()
{
  if (!standOnEnemy())
    {
    if (walkCounter++ < 4)
      { // Held fllt vier Positionen
  fallTimer->start(FALLDELAY,TRUE);
  { rely+=STEP; absy+=STEP;}
      } else //Held ist ein Feld weitergefallen
  { // Verschiebung der Figur zum 0-Punkt des Objekts (Brick etc...)
    rely = 0; heroy = ++ypos; absy=objectSize*ypos;// wird Null  und Figur eins runter gesetzt.
    collectNugget(); // Zeit evtl. Nugget zu nehmen
    if (!(canStand()||hangAtPole())) // Held muss wohl weiterfallen...
      { fallTimer->start(FALLDELAY,TRUE);
      walkCounter=1;
      } else { // Held hat Boden (oder Feind) unter den Fssen
        status_ = KGrObj::STANDING; // oder hngt an Stange -> steh!
        walkTimer->start(WALKDELAY,TRUE);
        direction_ = (actualPixmap == 19) ? KGrObj::RIGHT : KGrObj::LEFT; 
        if (map->playField[xpos][ypos]->whatIam() == POLE)
          actualPixmap = (direction_ == KGrObj::RIGHT) ? 8 : 12;
        else 
          actualPixmap = (direction_ == KGrObj::RIGHT) ? 0 : 4;
      }
  }
    showFigure();
    } else fallTimer->start(FALLDELAY,TRUE);
}


void KGrHero::showFigure()
{

  eraseOldFigure();
  
  map->playField[xpos][ypos]->drawFigure(heroPm[actualPixmap], relx, rely);

  if (relx != 0)
  {
    if (relx < 0)
    {
      map->playField[xpos - 1][ypos]->drawFigure(heroPm[actualPixmap], relx + objectSize, rely);
    }
    else
    {
      map->playField[xpos + 1][ypos]->drawFigure(heroPm[actualPixmap], relx - objectSize, rely);
    }
  }
  else if (rely != 0)
  {
    if (rely < 0)
    {
      map->playField[xpos][ypos - 1]->drawFigure(heroPm[actualPixmap], relx, rely + objectSize);
    }
    else
    {
      map->playField[xpos][ypos + 1]->drawFigure(heroPm[actualPixmap], relx, rely - objectSize);
    }
  }

  // Merke alte Werte zum lschen der Figur
  mem_x = xpos;
  mem_y = ypos;
  mem_relx = relx;
  mem_rely = rely;
}

void KGrHero::dig()
{
  int t=1;
  
  if(status_ != KGrObj::STANDING)
    t=2;
    
  if( direction_ == KGrObj::LEFT && 
      xpos-t >= 0 )
  {
    switch( map->playField[xpos-t][ypos]->whatIam() )
    {
      case HLADDER:
      case FREE:
      case HOLE:
        if( map->playField[xpos-t][ypos+1]->whatIam() == BRICK )
          ((KGrBrick*)map->playField[xpos-t][ypos+1])->dig();
        break;
      default :
        break;
    }
  }
  else if( direction_ == KGrObj::RIGHT && 
           xpos+t < map->objectsX )
  {
    switch( map->playField[xpos+t][ypos]->whatIam() )
    {
      case HLADDER:
      case FREE:
      case HOLE:
        if( map->playField[xpos+t][ypos+1]->whatIam() == BRICK )
          ((KGrBrick*)map->playField[xpos+t][ypos+1])->dig();
        break;
      default :
        break;
    }
  }
}

void KGrHero::setEnemyList(EnemiesList *e)
{
  enemies = e;
}

bool KGrHero::standOnEnemy()
{
  KGrEnemy * enemy;
  
  for(EnemiesListIterator it(*enemies); it.current(); ++it)
  {
    enemy = it.current();
  
    // um eine gewisse Tolleranz
    // zu haben diese Spanne von 4
    if( (KGrObj::CAPTIVE == enemy->status()) &&
        (absx == enemy->getx()) && 
        ((absy+objectSize-STEP) <= enemy->gety()) )
      return true;
  }
  
  return false;
}

void KGrHero::collectNugget()
{
  if( map->playField[xpos][ypos]->whatIam() == NUGGET )
  {
    ((KGrFree *)map->playField[xpos][ypos])->setNugget(false);
//    showFigure();
    
    // sendet der Application dass alle Nuggets gesammelt
    if( !(--nuggets) )
      haveAllNuggets();
      
    //  sind, um versteckte Leitern zu zeigen
    gotNugget(); // sendet der Application ein Nugget um Score zu erhhen
  }
}

bool KGrHero::isInEnemy()
{
  KGrEnemy * enemy;
  
  for(EnemiesListIterator it(*enemies); it.current(); ++it)
  {
    enemy = it.current();
    
    if( isInside(enemy->getx(), enemy->gety()) ||
        isInside(enemy->getx() - objectSize + 1, enemy->gety())||
        isInside(enemy->getx(), enemy->gety() - objectSize + 1))
      return true;
  }
    
  return false;
}

bool KGrHero::isInside(int enemyx, int enemyy)
{
 return ( (absx >= enemyx) &&
          (absx <= enemyx + objectSize - 1) &&
          (absy >= enemyy) &&
          (absy <= enemyy + objectSize - 1) );
}



/*******************************************************
                        _____                            
                       | ____|_ __   ___ _ __ ___  _   _ 
                       |  _| | '_ \ / _ \ '_ ` _ \| | | |
                       | |___| | | |  __/ | | | | | |_| |
                       |_____|_| |_|\___|_| |_| |_|\__, |
                                                   |___/
 
********************************************************/
KGrEnemy :: KGrEnemy (int xpos, int ypos)
  : KGrFigure(xpos, ypos)
{
  QString name;
  
  actualPixmap = 18;
  WALKDELAY = 55;
  FALLDELAY = 100;
  CAPTIVEDELAY = 500;

  captiveCounter = 0;

  searchStatus = HORIZONTAL;

  birthX=xpos;
  birthY=ypos;

    
  for (int i = 0; i < 20; i++)
  {
    name = QString("enemy1_") + QString::number(i);
    enemyPm[0][i] = QIconLoader::instance()->image( name );
    if( enemyPm[0][i].isNull() )
      qDebug("Hmm, enemy1 error!");
        
    name = QString("enemy2_") + QString::number(i);
    enemyPm[1][i] = QIconLoader::instance()->image( name );
    if( enemyPm[1][i].isNull() )
      qDebug("Hmm, enemy2 error!");
  }
  
  captiveTimer = new QTimer (this);
    
  connect (captiveTimer, SIGNAL(timeout()), SLOT(captiveTimeDone()));
  connect (walkTimer, SIGNAL (timeout ()), SLOT (walkTimeDone ()));
  connect (fallTimer, SIGNAL (timeout ()), SLOT (fallTimeDone ()));
}

KGrEnemy :: ~KGrEnemy ()
{
  delete captiveTimer;
  delete walkTimer;
  delete fallTimer;
}

void KGrEnemy::init(int a,int b)
{
  walkTimer->stop(); // alles stoppen bevor die Werte neu gesetzt
  fallTimer->stop(); // werden, da es sonst zu ungewollten Effekten 
  captiveTimer->stop(); // kommen kann

  eraseOldFigure();
  
  xpos = mem_x = a;
  ypos = mem_y = b;
  relx = mem_relx = 0;
  rely = mem_rely = 0;

  absx=objectSize*xpos;
  absy=objectSize*ypos;

  actualPixmap = 19;

  status_ = KGrObj::STANDING;
}

void KGrEnemy::walkTimeDone ()
{
  if( true /*!isInEnemy()*/ )
  {
    switch (direction_)
    {
      case KGrObj::UP:  
        if(map->playField[xpos][ypos]->whatIam() == BRICK)
        {// sollte er aber in einem Brick sein, dann
          xpos=birthX;
          ypos=birthY;
          relx=0;
          rely=0;
          startSearching();
        } // stirbt er wohl
        else
        {
          walkUp();
          if( (rely==0) && (map->playField[xpos][ypos+1]->whatIam() == USEDHOLE) ) // Enemy kletterte grad
            ((KGrBrick *)map->playField[xpos][ypos+1])->unUseHole();// aus einem Loch hinaus-> gib es frei!
        }
        break;
      case KGrObj::DOWN:
        walkDown();
        break;
      case KGrObj::RIGHT:
        walkRight();
        break;
      case KGrObj::LEFT:
        walkLeft();
        break;
      default:
        searchStatus = (searchStatus == VERTIKAL) ? HORIZONTAL : VERTIKAL;
        status_ = KGrObj::STANDING;
        break;
      }
    // wenn die Figur genau ein Feld gelaufen ist
    if (status_ == KGrObj::STANDING) // dann suche den Helden
      { direction_ = searchbestway(xpos,ypos,herox,heroy); // und
      status_ = KGrObj::WALKING; // initialisiere die Zhlervariablen und
      walkCounter = 1; // den Timer um den Held weiter
      walkTimer->start (WALKDELAY, TRUE); // zu jagen
      if (!nuggets)
        collectNugget();
      else
        dropNugget();
      }     
  }
  else
    walkTimer->start (WALKDELAY, true);
    
  showFigure();
}

void KGrEnemy::fallTimeDone ()
{
  if (map->playField[xpos][ypos+1]->whatIam() == HOLE){ // wenn Enemy ins Loch fllt
    ((KGrBrick*)map->playField[xpos][ypos+1])->useHole(); // reserviere das Loch, damit kein anderer es benutzt
    if( nuggets){  //und er muss Gold vorher fallen lassen
      ((KGrFree *)map->playField[xpos][ypos])->setNugget(true);
      nuggets=0;      
    } 
  }
  if (map->playField[xpos][ypos]->whatIam()==BRICK) {// sollte er aber in einem Brick sein, dann
    xpos=birthX; ypos=birthY; relx=0;rely=0; startSearching();} // stirbt er wohl
  
  if (walkCounter++ < 4){
    fallTimer->start(FALLDELAY,TRUE);
    { rely+=STEP; absy+=STEP;}
  } else { rely = 0; ypos ++; absy=objectSize*ypos;
  if (map->playField[xpos][ypos]->whatIam() == USEDHOLE)
    {
      status_ = KGrObj::CAPTIVE;
      captiveTimer->start(CAPTIVEDELAY,TRUE);
    }else if (!(canStand()||hangAtPole()))
      {
        fallTimer->start(FALLDELAY,TRUE);
        walkCounter=1;
      } else {
        status_ = KGrObj::STANDING;
        if (hangAtPole())
          actualPixmap = (direction_ == KGrObj::RIGHT) ? 8 : 12;
      }
  }
  if (status_ == KGrObj::STANDING)
    {status_ = KGrObj::WALKING;
    walkCounter = 1;
    direction_ = searchbestway(xpos,ypos,herox,heroy);
    walkTimer->start (WALKDELAY, TRUE);
    if (!nuggets)
      collectNugget();
    else
      dropNugget();
    } 
  showFigure();
}

void KGrEnemy::captiveTimeDone()
{
  if (map->playField[xpos][ypos]->whatIam()==BRICK)
    { xpos=birthX; ypos=birthY; relx=0;rely=0;startSearching();}
  else 
    if (captiveCounter > 6){
      status_ = KGrObj::WALKING;
      walkCounter = 1;
      direction_ = KGrObj::UP;
      walkTimer->start (WALKDELAY, TRUE);
      captiveCounter = 0;
    } else {
      if(!isInEnemy())
        captiveCounter ++;
      captiveTimer->start(CAPTIVEDELAY,TRUE);
      showFigure();
    }
}

void KGrEnemy::startSearching()
{
  init(xpos,ypos);
  if (canStand()||(map->playField[xpos][ypos+1]->whatIam()==USEDHOLE))
    {status_ = KGrObj::WALKING;
    walkTimer->start (WALKDELAY, TRUE);}
  else
    {status_ = KGrObj::FALLING;
    fallTimer->start (FALLDELAY, TRUE);}

  walkCounter = 1;
  direction_ = searchbestway(xpos,ypos,herox,heroy);
}

void KGrEnemy::collectNugget()
{
  if ((map->playField[xpos][ypos]->whatIam() == NUGGET) &&
      ((int)(5.0*rand()/RAND_MAX+1.0) > 4) && (nuggets==0)){ 
    ((KGrFree *)map->playField[xpos][ypos])->setNugget(false);
    nuggets=1;
  }
}

void KGrEnemy::dropNugget()
{
  if (((int)(DROPNUGGETDELAY*rand()/RAND_MAX+1.0) > DROPNUGGETDELAY-5) &&
      (map->playField[xpos][ypos]->whatIam() == FREE))    { 
    ((KGrFree *)map->playField[xpos][ypos])->setNugget(true);
    nuggets=0;
  }
}

void KGrEnemy::showFigure ()
{
  eraseOldFigure ();
  map->playField[xpos][ypos]->drawFigure (enemyPm[nuggets][actualPixmap], relx, rely);
  if (relx != 0)
    if (relx < 0)
      map->playField[xpos-1][ypos]->drawFigure (enemyPm[nuggets][actualPixmap], relx + objectSize, rely);
    else
      map->playField[xpos+1][ypos]->drawFigure (enemyPm[nuggets][actualPixmap], relx - objectSize, rely);
  else if (rely != 0)
    if (rely < 0)
      map->playField[xpos][ypos-1]->drawFigure (enemyPm[nuggets][actualPixmap], relx, rely + objectSize);
    else
      map->playField[xpos][ypos+1]->drawFigure (enemyPm[nuggets][actualPixmap], relx, rely - objectSize);

  // Merke alte Werte zum lschen der Figur
  mem_x = xpos;
  mem_y = ypos;
  mem_relx = relx;
  mem_rely = rely;
}

bool KGrEnemy::canWalkUp()
{
  return ((map->playField[xpos][ypos-1]->whatIam() != BRICK) &&
	  (map->playField[xpos][ypos-1]->whatIam() != BETON) &&
	  (map->playField[xpos][ypos-1]->whatIam() != FBRICK) &&
	  ((map->playField[xpos][ypos]->whatIam() == USEDHOLE) ||
	   (map->playField[xpos][ypos]->whatIam() == LADDER)));
}

KGrObj::Direction KGrEnemy::searchbestway(int ew,int eh,int hw,int hh)
{
  if(searchStatus==VERTIKAL){
    if (eh > hh) 
      return searchupway(ew,eh,hw,hh);
    if (eh < hh)
      return searchdownway(ew,eh,hw,hh);
    return KGrObj::STAND;
  } else {
    if (ew > hw) 
      return searchleftway(ew,eh,hw,hh);
    if (ew < hw)
      return searchrightway(ew,eh,hw,hh);
    return KGrObj::STAND;
  }
}

KGrObj::Direction KGrEnemy::searchdownway(int ew,int eh,int hw, int hh)
{
  int i,j;
  i=j=ew; 
  if( map->playField[ew][eh]->searchValue & CANWALKDOWN )
    return KGrObj::DOWN;
  else 
    while((i>=0)||(j<=FIELDWIDTH))
    {
   if (i>=0)
     if ( map->playField[i][eh]->searchValue & CANWALKDOWN)
       return KGrObj::LEFT;
     else 
       if (!( map->playField[i--][eh]->searchValue & CANWALKLEFT))
       i=-1;
   if (j<=FIELDWIDTH)
     if ( map->playField[j][eh]->searchValue & CANWALKDOWN)
       return KGrObj::RIGHT;
     else 
       if (!( map->playField[j++][eh]->searchValue & CANWALKRIGHT))
       j=FIELDWIDTH+1;
 }
 return KGrObj::STAND;
}

KGrObj::Direction KGrEnemy::searchupway(int ew,int eh,int hw, int hh)
{
  int i,j;
  i=j=ew; 
  if( map->playField[ew][eh]->searchValue & CANWALKUP)
    return KGrObj::UP;
 else while((i>=0)||(j<=FIELDWIDTH)){// search for the first way up
   if (i>=0)
     if ( map->playField[i][eh]->searchValue & CANWALKUP)
       return KGrObj::LEFT;
     else 
       if (!( map->playField[i--][eh]->searchValue & CANWALKLEFT))
       i=-1;
   if (j<=FIELDWIDTH)
     if ( map->playField[j][eh]->searchValue & CANWALKUP)
       return KGrObj::RIGHT;
     else 
       if (!( map->playField[j++][eh]->searchValue & CANWALKRIGHT))
	 j=FIELDWIDTH+1;
 }
 return KGrObj::STAND;
}

KGrObj::Direction KGrEnemy::searchleftway(int ew,int eh,int hw, int hh)
{
  int i,j;
  i=j=eh; 
  if ( map->playField[ew][eh]->searchValue & CANWALKLEFT) /* kann figur nach links laufen ?*/
    return KGrObj::LEFT;
 else while((i>=0)||(j<=FIELDHEIGHT)){ /* in den grenzen ?*/
   if (i>=0)
     if ( map->playField[ew][i]->searchValue & CANWALKLEFT) /* ein weg nach links- oben gefunden ?*/
       return KGrObj::UP; /* geh nach oben */
     else 
       if (!( map->playField[ew][i--]->searchValue & CANWALKUP)) /* sonst ...*/
	 i=-1;
   if (j<=FIELDHEIGHT) 
     if ( map->playField[ew][j]->searchValue & CANWALKLEFT) /* ein weg nach links- unten gefunden ?*/
       return KGrObj::DOWN; /* geh nach unten */
     else 
       if (!( map->playField[ew][j++]->searchValue&CANWALKDOWN)) /* sonst ... */
       j=FIELDHEIGHT+1;
 }
 return KGrObj::STAND; /* default */
} 

KGrObj::Direction KGrEnemy :: searchrightway(int ew,int eh,int hw, int hh)
{
  int i,j;
  i=j=eh; 
  if ( map->playField[ew][eh]->searchValue & CANWALKRIGHT)
    return KGrObj::RIGHT;
  else while((i>=0)||(j<=FIELDHEIGHT)){
    if (i>=0)
      if ( map->playField[ew][i]->searchValue & CANWALKRIGHT)
	return KGrObj::UP;
      else 
	if (!( map->playField[ew][i--]->searchValue & CANWALKUP))
	i=-1;
    if (j<=FIELDHEIGHT)
      if ( map->playField[ew][j]->searchValue & CANWALKRIGHT)
	return KGrObj::DOWN;
      else 
	if (!( map->playField[ew][j++]->searchValue & CANWALKDOWN))
	j=FIELDHEIGHT+1;
  }
  return KGrObj::STAND;
}

void KGrEnemy::setEnemyList(EnemiesList *e)
{
  enemies = e;
}

bool KGrEnemy::standOnEnemy()
{
  KGrEnemy * enemy;
  
  for(EnemiesListIterator it(*enemies); it.current(); ++it)
  {
    enemy = it.current();
    
    if( (absx == enemy->getx()) && ((absy+objectSize-STEP) <= enemy->gety()) )
      return true;
  }
  
  return false;
}

bool KGrEnemy::isInside(int enemyx, int enemyy)
{ 
  return( (absx >= enemyx) &&
          (absx <= enemyx + objectSize - 1) &&
          (absy >= enemyy) &&
          (absy <= enemyy + objectSize - 1) );
}

bool KGrEnemy::isInEnemy()
{
  KGrEnemy * enemy = 0L;
  
  for(EnemiesListIterator it(*enemies); it.current(); ++it)
  {
    enemy = it.current();
    
    if( enemy->enemyId >= enemyId )
      break;
      
    if( isInside(enemy->getx(), enemy->gety()) ||
        isInside(enemy->getx() - objectSize + 1, enemy->gety()) )
      return true;
  }
  
  return false;
}

