/***************************************************************************
                          decorobjects.cpp  -  description
                             -------------------
    begin                : Tue Jul 30 2002
    copyright            : (C) 2002 by Eugen C.
    email                : eug@thekompany.com
 ***************************************************************************/

#include "decorobjects.h"
#include "qiconloader.h"

#include <qtimer.h>

KGrFree :: KGrFree (bool nug, QWidget * parent, const char *name)
  : KGrObj (parent, name)
{
  setNugget(nug);
}

void KGrFree :: setNugget(bool nug)
{
  if (!nug)
    {
      setBackgroundPixmap( QIconLoader::instance()->image("hgbrick.xpm") );
      rtti = FREE;
    }
  else
    {
      setBackgroundPixmap( QIconLoader::instance()->image("nugget.xpm") );
      rtti = NUGGET;
    }
}

KGrFree :: ~KGrFree ()
{
}

PseudoHero::PseudoHero(QWidget *parent, const char *name)
  :KGrObj (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image("pseudohero.png") );
  rtti = HERO;
}

PseudoHero::~PseudoHero()
{
}

PseudoEnemy::PseudoEnemy(QWidget *parent, const char *name)
  :KGrObj (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image("pseudoenemy.png") );
  rtti = ENEMY;
}

PseudoEnemy::~PseudoEnemy()
{
}

PseudoLadder::PseudoLadder(QWidget *parent, const char *name)
  :KGrObj (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image("pseudoladder.png") );
  rtti = HLADDER;
}

PseudoLadder::~PseudoLadder()
{
}

KGrPole :: KGrPole (QWidget * parent, const char *name)
  :KGrObj (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image("pole.xpm") );
  rtti = POLE;
}

KGrPole :: ~KGrPole ()
{
}

KGrStones :: KGrStones (QWidget * parent, const char *name)
  :KGrObj (parent, name)
{
  blocker = true;
}

KGrStones :: ~KGrStones () 
{
}

/* +++++++++++++++ BRICK ++++++++++++++++ */

KGrBrick :: KGrBrick (QWidget * parent, const char *name)
  : KGrStones (parent, name),
    timer(0L)
{
  QPixmap brick( QIconLoader::instance()->image("brick.xpm") );
  
  if( brick.isNull() )
  {
    qDebug("Hmm, cannot load the brick icon.");
  }
  else
  {
    QString name;
    
    if( QIconLoader::instance()->image("brick_0").isNull() )
    {
      for (int i = 0; i < 10; i++)
      {
        QPixmap * pixmap = new QPixmap(objectSize, objectSize);
        bitBlt(pixmap, 0, 0, &brick, i * objectSize, 0, objectSize, objectSize);
        name = QString("brick_") + QString::number(i);
        QIconLoader::instance()->setImage(name, pixmap);
      }
    }
    
    for (int i = 0; i < 10; i++)
    {
      name = QString("brick_") + QString::number(i);
      digpix[i] = QIconLoader::instance()->image( name );
      if( digpix[i].isNull() )
        qDebug("Hmm, brick error!");
    }
    
    setBackgroundPixmap(digpix[0]);
    
    // init timer
    timer = new QTimer (this);
    connect (timer, SIGNAL (timeout ()), SLOT (timeDone ()));
    rtti = BRICK;
  }
}

void KGrBrick::dig (void)
{
  dig_counter = 1;
  rtti = HOLE;
  setBackgroundPixmap (digpix[1]);
  timer->start (DIGDELAY, TRUE);
}

void KGrBrick::timeDone ()
{
  if (dig_counter < 9) {
    dig_counter++;
    if (dig_counter == 5) // the hole is competely digged
      timer->start (DIGDELAY * HOLETIME, TRUE); // so we need a longer delay
    else {
      timer->start (DIGDELAY, TRUE);
      if (dig_counter >= 8)
        rtti = BRICK;
    }
  } else dig_counter = 0;
  setBackgroundPixmap (digpix[dig_counter]);
}

void KGrBrick :: useHole(){
  if (rtti == HOLE)
    rtti = USEDHOLE;
}

void KGrBrick :: unUseHole(){
  if (rtti == USEDHOLE)
    rtti = HOLE;
}

KGrBrick :: ~KGrBrick ()
{
  if( timer )
  {
    delete timer;
    timer = 0L;
  }
}

KGrFbrick :: KGrFbrick (QWidget * parent, const char *name)
  :KGrStones (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image("fbrick.xpm") );
  rtti = FBRICK;
}

KGrFbrick :: ~KGrFbrick ()
{
}

KGrBeton :: KGrBeton (QWidget * parent, const char *name)
  :KGrStones (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image("beton.xpm") );
  rtti = BETON;
}

KGrBeton :: ~KGrBeton ()
{
}

KGrLadder :: KGrLadder (QWidget * parent, const char *name)
  :KGrObj (parent, name)
{
  setBackgroundPixmap (QIconLoader::instance()->image ("ladder.xpm"));
  rtti = LADDER;
}

KGrLadder :: ~KGrLadder ()
{
}

KGrHladder :: KGrHladder (QWidget * parent, const char *name)
  :KGrLadder (parent, name)
{
  setBackgroundPixmap( QIconLoader::instance()->image ("hgbrick.xpm") );
  rtti = HLADDER;
}

void KGrHladder::showLadder()
{
  rtti = LADDER;
  setBackgroundPixmap( QIconLoader::instance()->image("ladder.xpm") );
}

KGrHladder :: ~KGrHladder ()
{
}
