#ifndef _DIRS_H
#define _DIRS_H

#include <defines.h>

#ifdef _QTE_
#include <qpe/qpeapplication.h>
#include <qpe/resource.h>
#endif
#include <qstringlist.h>

/**
 * Handles the list of directories that may be searched for installed data.
 */
class Dirs
{
  public:

    /**
     * Find a file in the standard paths. Returns the first filename which
     * matches while searching the list in order.
     *
     * To find using a relative path, just prepend the path.
     *
     * Example:
     *
     * QString path = Dirs::locate("icons/bigpic.png");
     *
     * @return QString::null if file is not found, otherwise full path
     * to file.
     */
    static QString locate(const QString & filename);

    /**
     * Add a path to the search list, before all the others. Files in
     * this directory will be located before others.
     */
    static void prependPath(const QString & fullPath);

    /**
     * Add a path to the search list, before all the others. Files in
     * this directory will be located before others.
     */
    static void appendPath(const QString & fullPath);

    /**
     * Remove a path from the search list. Use this when a plugin is
     * unloaded, for example.
     */
    static bool removePath(const QString & fullPath);

    /**
     * Have a peek at the path list. Probably useless, but no harm
     * providing it.
     */
    QStringList pathList() const;

  private:

    Dirs();
    Dirs(const Dirs &);
    Dirs & operator = (const Dirs &);
    ~Dirs();

    static Dirs * thisInstance()
    {
      if (0 == instance_)
        instance_ = new Dirs;

      return instance_;
    }

    static Dirs * instance_;
    QStringList pathList_;
};

#endif
