/***************************************************************************
                          fieldmap.h  -  description
                             -------------------
    begin                : Tue Jul 30 2002
    copyright            : (C) 2002 by Eugen C.
    email                : eug@thekompany.com
 ***************************************************************************/

#ifndef FIELDMAP_H
#define FIELDMAP_H

#include <qwidget.h>
#include <qpixmap.h>

#include <defines.h>

// X = Width, Y = High and SIZE = pics number
#define OBJECT_SIZE           16

#ifdef _QTE_
#define OBJECTS_X             16
#define OBJECTS_Y             18
#else
#define OBJECTS_X             30
#define OBJECTS_Y             22
#endif

const char FREE    = ' ';
const char ENEMY   = 'E';
const char HERO    = 'R';
const char BETON   = 'X';
const char BRICK   = 'M';
const char FBRICK  = 'F';
const char HLADDER = 'Z';
const char LADDER  = 'H';
const char NUGGET  = 'N';
const char POLE    = 'T';
const char HOLE    = 'O';
const char USEDHOLE= 'U';

const char CANWALKLEFT  = 0x1;
const char CANWALKRIGHT = 0x2;
const char CANWALKUP    = 0x4;
const char CANWALKDOWN  = 0x8;
const char VISITED      = 0x10;

#ifdef _QTE_
const char FIELDWIDTH   = 14;
const char FIELDHEIGHT    = 16;
#else
const char FIELDWIDTH   = 28;
const char FIELDHEIGHT    = 20;
#endif

const char VERTIKAL     = 0;
const char HORIZONTAL   = 1;

const double DROPNUGGETDELAY = 500.0;
const int DIGDELAY = 200;
const int HOLETIME = 40;
const int STEP = 2;
const int MAX_STEPS = OBJECT_SIZE / STEP;


/** The root of all the game objects.*/
class KGrObj : public QWidget
{ 
  Q_OBJECT
  
  public:
    enum Position {RIGHTWALK1, RIGHTWALK2, RIGHTWALK3, RIGHTWALK4,
                   LEFTWALK1, LEFTWALK2, LEFTWALK3, LEFTWALK4,
                   RIGHTCLIMB1, RIGHTCLIMB2, RIGHTCLIMB3, RIGHTCLIMB4,
                   LEFTCLIMB1, LEFTCLIMB2, LEFTCLIMB3, LEFTCLIMB4,
                   CLIMB1, CLIMB2,
                   FALL1, FALL2};
    enum Status {STANDING, FALLING, WALKING, CLIMBING, CAPTIVE};
    enum Direction {RIGHT, LEFT, UP, DOWN, STAND};

    KGrObj(QWidget *parent=0, const char *name=0);
    virtual ~KGrObj();
    
    virtual void drawFigure(QPixmap, int, int);
    virtual void eraseFigure(int, int);
    
    virtual void init(int, int);
    void setx(int);
    void sety(int);
    char whatIam();
    
    int searchValue;
    bool blocker; // Beton or Brick -> TRUE
    
  signals:
    void pressed(int, int);
    
  protected:
  
    virtual void paintEvent(QPaintEvent*);
    
    int xpos;
    int ypos;
    char rtti;
    int objectSize;
    QPixmap drawFigure_;
    int xDrawFigure_;
    int yDrawFigure_;
};

/** It is a pointer map of game objects.
  * @author Eugen C.
  */
class FieldMap
{
  public:
    FieldMap(int X, int Y);
    virtual ~FieldMap();
    
    KGrObj ***playField;
    int objectsX;
    int objectsY;
};

#endif
