#ifndef LOADRUNNER_H 
#define LOADRUNNER_H 

#include <qmainwindow.h>
#include <qcolor.h>
#include <qkeycode.h>
#include <qlist.h>
#include <qstring.h>

#include "fieldmap.h"
#include "decorobjects.h"
#include "animatedobjects.h"
#include "loadrunnerwidget.h"

class QStatusBar;
class LoadRunnerPrivate;

// Unique strings for QAction-s
#define ACTION_FILE_EXIT      "FileExit"
#define ACTION_GAME_NEW       "GameNew"
#define ACTION_GAME_SAVE      "GameSave"
#define ACTION_GAME_RELOAD    "GameReload"
#define ACTION_HELP_ABOUT     "HelpAbout"
#define ACTION_HELP_INDEX     "HelpIndex"

#define LIFES_DEFAULT         5

#define MAX_LEVELS_NUMBER     10

class LoadRunner : public QMainWindow
{
  Q_OBJECT
  
  public:
  
    enum State { Error=-1, NewGame=0, Running, Pause, Stop, Edit };
  
    LoadRunner(QWidget *parent = 0, const char *name = 0);
    virtual ~LoadRunner();
    
    virtual void keyPressEvent(QKeyEvent *);
    virtual void paintEvent(QPaintEvent *);
    void initStatusBar();
  
  public slots:
  
    virtual void slotFileExit();
    virtual void slotGameNew();
    virtual void slotGameSave();
    virtual void slotGameReload();
    virtual void slotHelpAbout();
    virtual void slotHelpIndex();
    virtual void slotChangeLifes(int);
    virtual void slotChangeScore(int);
    virtual void slotChangeLevel(int);
    virtual void slotIncrementScore();
  
  protected slots:
  
    void slotShowHidden();
    void slotStop();
    void slotNextLevel();
  
  protected:
    
    void buildActions();
  
  private:
  
    LoadRunnerPrivate * d;
};

#endif
