
#include <defines.h>
#include <dirs.h>
#include <qiconloader.h>

#ifdef _QTE_
#include <qpe/resource.h>
#endif

QIconLoader * QIconLoader::instance_ = 0L;

QIconLoader::QIconLoader()
{
  images_.setAutoDelete(true);

#ifdef _QTE_
  // Ex: /opt/Qtopia/pics/eLoadRunner/
  iconsPath_=QString("pics/") + AppName + "/";
#else
  // Ex: /usr/local/lib/tkc/eLoader/icons/
  iconsPath_="icons/";
#endif

}

QIconLoader::~QIconLoader()
{
}

  QPixmap
QIconLoader::barIcon(const QString & name)
{
//  qDebug("Try to find %s", name.latin1());
  
  QString path = Dirs::locate(iconsPath_ + name);

  if (path.isEmpty())
  {
    path = Dirs::locate(iconsPath_ + name + ".xpm");

    if (path.isEmpty())
    {
      path = Dirs::locate(iconsPath_ + name + ".png");
    }
  }

#ifdef _QTE_
  if( path.isEmpty() )
  {
    path = Resource::findPixmap(name);
  }
#endif
  
  return QPixmap(path);
}

QPixmap QIconLoader::loadIcon( const QString &name, int )
{
  return barIcon(name);
}

QString QIconLoader::iconPath( const QString &name, int )
{
  return Dirs::locate(iconsPath_ + name);
}

QIconSet QIconLoader::iconSet(const QString &name)
{
  QIconSet icons;
  QPixmap pixmap = barIcon(name);
  
  if( pixmap.isNull() )
  {
#ifdef _QTE_
    icons = Resource::loadIconSet(name);
#endif
  }
  else 
    icons.setPixmap(pixmap, QIconSet::Automatic, QIconSet::Normal);

  return icons;
}
    
void QIconLoader::setImage(const QString &name, QPixmap *pixmap)
{
  images_.insert(name, pixmap);
}
    
QPixmap QIconLoader::image(const QString &name)
{
  QPixmap * image = 0L;
  
  image = images_[name];
  if( !image )
  {
    QString path = Dirs::locate(iconsPath_ + name);
  
    if( path.isEmpty() )
    {
      path = Dirs::locate(iconsPath_ + name + ".xpm");
  
      if( path.isEmpty() )
      {
        path = Dirs::locate(iconsPath_ + name + ".png");
      }
    }
    
#ifdef _QTE_
    if( path.isEmpty() )
    {
      path = Resource::findPixmap(name);
    }
#endif

    if( !path.isEmpty() )
    {
#ifdef _QTE_
      image = new QPixmap(path, 0, QPixmap::DiffuseAlphaDither);
#else
      image = new QPixmap(path);
#endif
      
      if( !image->isNull() )
        images_.insert(name, image);
      else
      {
        qDebug("The image %s has a bad format.", path.latin1());
        
        delete image;
        image = 0L;
      }
    }
  }

  if( image )
    return *image;
  else
    return QPixmap();
}
    
// ---------- STATIC ----------------

QPixmap BarIcon(const QString & name)
{
  return QIconLoader::instance()->barIcon(name);
}

QPixmap DesktopIcon( const QString &name, int )
{
  return BarIcon(name);
}

QPixmap SmallIcon( const QString &name )
{
  return BarIcon(name);
}

QPixmap SmallIconSet( const QString &name )
{
  return BarIcon(name);
}

